#!/usr/bin/env python
#
# Copyright 2007 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import os
import cgi
import urllib
import datetime
import httplib2
import wsgiref.handlers
from google.appengine.ext import db
from google.appengine.api import users
from google.appengine.ext import webapp
from google.appengine.api import memcache
from google.appengine.ext.webapp import template
from google.appengine.ext.webapp.util import run_wsgi_app
from oauth2client.appengine import AppAssertionCredentials
from apiclient.discovery import build

# Global variables
datafile = "demo-bucket/language_id.txt"
model_id = "mymodel54" 
api_key = "AIzaSyDvTqvn9BC9_ZK3lO4KIUVbG3CiBvhLvT0"

# Set up the Prediction API service
credentials = AppAssertionCredentials(scope='https://www.googleapis.com/auth/prediction')
http = credentials.authorize(httplib2.Http(memcache))
service = build("prediction", "v1.4", http=http, developerKey=api_key)

class TrainModel(webapp.RequestHandler):
	def get(self):
		# train the model on the file
		self.response.out.write('Training the model. It may take several minutes for the training to complete.')
		payload = {'id': model_id, 'storageDataLocation': datafile }
		service.trainedmodels().insert(body=payload).execute()
	
class CheckModel(webapp.RequestHandler):
	def get(self):
		# checks if a model is trained
		self.response.out.write('Checking the status of the model.<br>')
		status = service.trainedmodels().get(id=model_id).execute()
		self.response.out.write('Status: ' + status['trainingStatus'])

def PredictLanguage(message):
    payload = {"input": {"csvInstance": [message] } }
    resp = service.trainedmodels().predict(id=model_id,body=payload).execute()
    prediction = resp['outputLabel']
    return cgi.escape(prediction)

def GetSentiment(message):
    """Returns true if the sentiment from the public prediction model is positive, false otherwise"""
    body = {'input' : {'csvInstance' : [message] }}
    output = service.hostedmodels().predict(body=body, hostedModelName='sample.sentiment').execute()
    prediction = output['outputLabel']
    if cgi.escape(prediction) == "positive":					  
        return True
    else:
	    return False

class Greeting(db.Model):
  """Models an individual Guestbook entry with an author, content, and date."""
  author = db.UserProperty()
  content = db.StringProperty(multiline=True)
  date = db.DateTimeProperty(auto_now_add=True)
  positive = db.BooleanProperty()
  language = db.StringProperty()


def guestbook_key(guestbook_name=None):
  """Constructs a datastore key for a Guestbook entity with guestbook_name."""
  return db.Key.from_path('Guestbook', guestbook_name or 'default_guestbook')


class MainPage(webapp.RequestHandler):
    def get(self):
        guestbook_name=self.request.get('guestbook_name')
        language = self.request.get('lang')
        # do something to get the correct language messages
        greetings_query = Greeting.all().ancestor(
            guestbook_key(guestbook_name)).order('-date')
        greetings = greetings_query.fetch(10)

        if users.get_current_user():
            url = users.create_logout_url(self.request.uri)
            url_linktext = 'Logout'
        else:
            url = users.create_login_url(self.request.uri)
            url_linktext = 'Login'
        template_values = {
            'greetings': greetings,
            'url': url,
            'url_linktext': url_linktext
        }

        path = os.path.join(os.path.dirname(__file__), 'index.html')
        self.response.out.write(template.render(path, template_values))


class Guestbook(webapp.RequestHandler):
  def post(self):
    guestbook_name = self.request.get('guestbook_name')
    language = self.request.get('lang')
    greeting = Greeting(parent=guestbook_key(guestbook_name))

    if users.get_current_user():
      greeting.author = users.get_current_user()

    greeting.content = self.request.get('content')
    greeting.positive = GetSentiment(greeting.content)
    greeting.language = PredictLanguage(greeting.content)

    greeting.put()
    self.redirect('/?' + urllib.urlencode({'guestbook_name': guestbook_name}))

application = webapp.WSGIApplication([
  ('/', MainPage),
  ('/sign', Guestbook),
	  ('/trainmodel', TrainModel),
	  ('/checkmodel', CheckModel)
], debug=True)


def main():
  run_wsgi_app(application)


if __name__ == '__main__':
  main()